/* AutoDoc Javascript */

var conf = {
  default_hash: "home",
  openapi: "/v1",
  pageinit: {
    home: "page_load_home",
    document: "page_load_document",
    upload: "page_load_upload",
    tags: "page_load_tags"
  },
  home: {
    width: 4,
    curpage: 0,
    end: 0
  },
  run: {
    "upload_handler": 0,
    "upload_status": 0
  },
  busy: 0
};

var temp = {
  upload: [],
  documentid: null
};

$(document).ready(function () {

  // loader icon thread
  loader_icon_update();

  // load the current page based on anchor
  change_page();

  // initialise drop zone
  upload_zone_init();

  // main page scroll event
  $(document).scroll(function () {
    page_home_scroll();
  });

  
  // document page
  // make primary button
  $('.autodoc_document_btn_primary').click(function(e) {
    $(e.target).addClass('disabled');
    var docid  = $('#autodoc_tab_document').data('docid');
    var pageid = $('.autodoc_document_image').data('pageid');

    loader_icon_on();
    $.ajax({
      type: "PATCH",
      url: conf.openapi + "/documents/" + docid + "?primarypage=" + pageid,
      success: function () {
        loader_icon_off();
        $('.autodoc_document_btn_primary').removeClass('disabled');
      }
    });

  });

  // tag menu select
  $('.autodoc_tag_list').change(function (e) {
    console.log(e);
    var tagId = $('.autodoc_tag_list').val();
    var tagName = $('.autodoc_tag_list').children("option:selected").html();
    var tagColor = $('.autodoc_tag_list').children("option:selected").attr('data');

    if (tagId == "") {
      $(".autodoc_tag_list").addClass('d-none');
      $(".autodoc_tag_text").removeClass('d-none');
      $('.autodoc_tag_text').val('');
      $('.autodoc_tag_color.btn-primary').click();
      $('.autodoc_tag_id').val('');
    }
    else {
      $(".autodoc_tag_list").addClass('d-none');
      $(".autodoc_tag_text").removeClass('d-none');
      $('.autodoc_tag_text').val(tagName);
      $('.autodoc_tag_color.btn-' + tagColor).click();
      $('.autodoc_tag_id').val(tagId);
      $('.autodoc_tag_text').keyup();
    }
  });

  // tags add button
  $('.autodoc_tag_add').click(function (e) {
    var tagId = $('.autodoc_tag_id').val();
    var tagName = $('.autodoc_tag_text').val();
    var tagColor = $('.autodoc_tag_color.active').attr('data').split('_').pop();
    var url = conf.openapi + "/tags/";
    var urlparam = "tagName=" + encodeURIComponent(tagName) + "&tagColor=" + encodeURIComponent(tagColor);
    var method = "POST";

    if (tagId != "") {
      method = "PATCH";
      url += tagId;
    }

    loader_icon_on();
    $.ajax({
      type: method,
      url: url + '?' + urlparam,
      success: function () {
        loader_icon_off();
        location.reload();
      }
    });
  });

  // tags delete event
  $('.autodoc_tag_delete').click(function (e) {
    var tagId = $('.autodoc_tag_id').val();

    if (tagId != "") {
      loader_icon_on();
      $.ajax({
        type: "DELETE",
        url: conf.openapi + "/tags/" + tagId,
        success: function () {
          loader_icon_off();
          location.reload();
        }
      });
    }
    else {
      location.reload();
    }
  });

  // tags add color events
  $('.autodoc_tag_color').click(function (e) {
    var btn = e.target;
    while ($(btn).prop("tagName") != "BUTTON") {
      btn = $(btn).parent();
    }

    $(btn).parent().children().removeClass('active');
    $(btn).addClass('active');

    $('.autodoc_tag_btn_on').addClass('d-none');
    $('.autodoc_tag_btn_off').removeClass('d-none');

    $(btn).children('.autodoc_tag_btn_on').removeClass('d-none');
    $(btn).children('.autodoc_tag_btn_off').addClass('d-none');


    $(btn).parent().children().each(function (id, x) {
      var myclass = 'badge-' + $(x).attr('data').split('_').pop();
      $('#autodoc_tag_result').removeClass(myclass);
    });

    var myclass = 'badge-' + $(btn).attr('data').split('_').pop();
    $('#autodoc_tag_result').addClass(myclass);

  });

  // tag add text events
  $('.autodoc_tag_text').keyup(function (e) {
    $('#autodoc_tag_result').html($(e.target).val());
  });
});


// anchor change event
$(window).bind("hashchange", function () {
  change_page();
});

function change_page() {
  var args = document.location.hash.substr(1).split('_') || conf.default_hash.split('_');
  var hash = args.shift();

  if (!$("#autodoc_tab_" + hash).length) {
    hash = conf.default_hash;
  }

  /* change menu highlights and content visibility */
  $("#autodoc_navbar")
    .find("a")
    .each(function (id, obj) {
      var curhash = $(obj)
        .attr("href")
        .substr(1);
      if (curhash == hash) {
        $(obj)
          .parent()
          .addClass("active");
        $("#autodoc_tab_" + curhash).removeClass("d-none");
      } else {
        $(obj)
          .parent()
          .removeClass("active");
        $("#autodoc_tab_" + curhash).addClass("d-none");
      }
    });

  if (conf.pageinit[hash]) {
    var func = conf.pageinit[hash];
    console.log(func);
    if (window[func]) {
      window[func](args);
    }
  }
}

function isvisible(obj) {
  var top_of_fileent = $(obj).offset().top;
  var bottom_of_fileent = $(obj).offset().top + $(obj).outerHeight();
  var bottom_of_screen = $(window).scrollTop() + $(window).innerHeight();
  var top_of_screen = $(window).scrollTop();

  if (bottom_of_screen > top_of_fileent && top_of_screen < bottom_of_fileent) {
    return true;
  }
  return false;
}

function page_load_document(args) {
  var [docid] = args;

  if (!docid) { return false; }

  $('#autodoc_tab_document').data('docid', docid);

  $('.autodoc_document_empty').addClass('d-none');
  $('.autodoc_document_body').removeClass('d-none');

  $('.autodoc_document_image').attr('src','');

  loader_icon_on();
  $.getJSON(
    conf.openapi + "/documents/" + docid,
    function (doc) {
      loader_icon_off();
      $('span.autodoc_document_name').html(doc.name);
      $('input.autodoc_document_name').val(doc.name);
      
      $('.autodoc_document_created').html(doc.created);
      $('.autodoc_document_languages').html(doc.languages.join(','));
      $('.autodoc_document_owner').html(doc.owner);
      $('.autodoc_document_pages').html(doc.pageId.length);
      for(var i=0; i<doc.tags.length; i++) {
        var tag = $('.autodoc_document_tags_template').clone().removeClass('d-none');
        $(tag).html(doc.tags[i].name);
        $(tag).addClass('badge-' + doc.tags[i].color);
        $('.autodoc_document_tags').append(tag);
      }

      if ($('.slick-slide').length > 0) {
        $('.autodoc_document_carousel').slick('unslick');
      }
      $('.autodoc_document_carousel').children().remove();

      for (var i = 0; i < doc.pageId.length; i++) {
        var item = $('.autodoc_document_template_carousel').children().clone();
        $(item).children('img').attr('data-lazy',
          conf.openapi + "/pages/" + doc.pageId[i] + "/image?maxWidth=300");

        $(item).data('pageid', doc.pageId[i]);
        $(item).click(function(e) {
          var pageid = $(e.target).parent().data('pageid');
          $('.autodoc_document_image').attr('src',
            conf.openapi + "/pages/" + pageid + "/image"
          );
          $('.autodoc_document_image').data('pageid', pageid);
        });
        $('.autodoc_document_carousel').append(item);

      }
      $('.autodoc_document_image').attr('src', conf.openapi + "/pages/" + doc.primaryPage + "/image");
      $('.autodoc_document_image').data('pageid', doc.primaryPage);

      $('.autodoc_document_carousel').slick({
        lazyLoad: 'ondemand',
        slidesToShow: 3,
        slidesToScroll: 3,
        dots: true,
        infinite: true
      });

    }
  );
}

function page_load_upload() {
  upload_status();
}

function page_load_tags() {
  // update tag list
  loader_icon_on();
  $.getJSON(conf.openapi + "/tags", function (tags) {
    loader_icon_off();
    $.each(tags, function (n, tag) {
      $('.autodoc_tag_list').append(
        '<option data="' + tag.color + '" value="' + tag.id + '">' +
        tag.name +
        '</option>'
      );
    });
  });
}

function page_load_home(filter) {
  if (conf.home.curpage == 0) {
    $("#autodoc_home_body").html("");
  }

  if ( ! filter ) {
    filter = '';
  }

  var tmpl = $("#autodoc_template_home");
  var row = $(tmpl)
    .find(".autodoc_template_home_row")
    .clone();

  loader_icon_on();
  $.getJSON(
    conf.openapi +
    "/documents?pageSize=" +
    conf.home.width +
    "&pageIndex=" +
    conf.home.curpage +
    "&filter=" +
    encodeURIComponent(filter),
    null,
    function (data, textStatus, jqXHR) {
      // last document was loaded, stop scrolling
      loader_icon_off();
      if (data.length < conf.home.width) {
        conf.home.end = 1;
      }
      $.each(data, function (id, doc) {
        var col = $(tmpl)
          .find(".autodoc_template_home_col")
          .clone();
        $(col)
          .find(".autodoc_template_home_img")
          .attr(
            "src",
            conf.openapi +
            "/documents/" +
            doc.id +
            "/image?maxWidth=" +
            Math.floor(1000 / conf.home.width)
          );
        $(col).find(".autodoc_template_home_img").data("documentId", doc.id);
        $(col).find(".autodoc_template_home_img").on('click', function (e) {
          window.location.hash = 'document_' + $(e.target).data("documentId");
          e.preventDefault();
        });

        $(col)
          .find(".autodoc_template_home_owner")
          .html(doc.owner);

        $(col)
          .find(".autodoc_template_home_created")
          .html(doc.created);

        $(col)
          .find(".autodoc_template_home_name")
          .html(doc.name);

        $(col)
          .find(".autodoc_template_home_pages")
          .html(doc.pageId.length);

        $.each(doc.tags, function (id, tag) {
          $(col)
            .find(".autodoc_template_home_tag")
            .after(
              $(col)
                .find(".autodoc_template_home_tag")
                .clone()
                .html(tag)
            );
        });
        $(row).append(col);
      });
      $("#autodoc_home_body").append(row);
      // continue to load until out of browser.
      page_home_scroll();
    }
  );
}

function page_home_scroll() {
  // don't scroll if we've loaded the last document
  if (conf.home.end) {
    return;
  }
  if (isvisible($(".autodoc_template_home_col").last())) {
    console.log("scroll to page " + conf.home.curpage);
    conf.home.curpage++;
    // page_load_home();
    change_page();
  }
}

function upload_zone_init() {
  $(window).on("dragover dragleave drop", function (e) { e.preventDefault() });

  $('.autodoc_upload_zone').on({
    dragenter: upload_zone_highlight,
    dragleave: upload_zone_normal,
    dragover: upload_zone_over,
    drop: upload_zone_drop
  });
}

function upload_zone_highlight(e) {
  $(e.target).addClass('autodoc_upload_zone_hover');
}

function upload_zone_over(e) {
  return true;
}

function upload_zone_normal(e) {
  $(e.target).removeClass('autodoc_upload_zone_hover');
}

function upload_zone_drop(e) {
  var ev = e.originalEvent;

  $.each(ev.dataTransfer.files, function (id, file) {
    console.log(file);
    var fileid = temp.upload.length;
    temp.upload.push({
      progress: 0,
      error: null,
      obj: file
    });
    upload_zone_normal(e);
  });

  if (conf.run.upload_handler == 0) {
    upload_handler();
  }

  if (conf.run.upload_status == 0) {
    upload_status();
  }

}

function upload_handler() {
  var stop = 1;
  conf.run.upload_handler = 1;
  console.log("upload_handler");
  if (temp.documentid == null) {
    console.log("creating documentId");
    loader_icon_on();
    $.post(conf.openapi + "/documents", {}, function (data) {
      loader_icon_off();
      temp.documentid = data.id;
    });
    stop = 0;
  }
  else {

    for (var id = 0; id < temp.upload.length; id++) {
      var file = temp.upload[id];
      if (file.progress != 100) {
        if (file.progress == 0) {
          if (file.obj.type != 'application/pdf' && file.obj.type != 'image/png' && file.obj.type != 'image/jpeg') {
            file['progress'] = 100;
            file['error'] = 'invalid file type';
          }
          else {
            var reader = new FileReader();
            file['progress'] = 1;

            $(reader).on('load', id, upload_read_event);
            $(reader).on('loadstart', id, upload_read_event);
            $(reader).on('loadend', id, upload_read_event);
            $(reader).on('progress', id, upload_read_event);
            $(reader).on('error', id, upload_read_event);
            $(reader).on('abort', id, upload_read_event);

            reader.readAsArrayBuffer(file.obj);
          }
        }
        stop = 0;
        break;
      }
    }
  }
  if (stop) {
    conf.run.upload_handler = 0;
  }
  else {
    setTimeout(function () { upload_handler(); }, 500);
  }
}

function upload_read_event(e) {
  if (e.type == 'progress') {
    if (e.lengthComputable) {
      var progress = Math.floor(e.originalEvent.loaded / e.originalEvent.total * 50);
      if (progress < 1) { progress = 1; }
      if (progress > 49) { progress = 49; }
      temp.upload[e.data]['progress'] = progress;
    }
  }
  else if (e.type == 'load') {

    temp.upload[e.data]['progress'] = 50;

    loader_icon_on();
    $.ajax({
      type: 'POST',
      url: conf.openapi + "/documents/" + temp.documentid + "/data",
      data: e.target.result,
      contentType: temp.upload[e.data].obj.type,
      context: { id: e.data },
      processData: false,
      xhr: function () {
        var xhr = $.ajaxSettings.xhr();
        xhr.upload.addEventListener("progress", (function () {
          var id = e.data;
          return function (evt) {
            if (evt.lengthComputable) {
              var progress = Math.floor(evt.loaded / evt.total * 50);
              if (progress < 1) { progress = 1; }
              if (progress > 49) { progress = 49; }
              temp.upload[id].progress = 50 + progress;
              // console.log("upload_progress_event", evt);
            }
          }
        })(), false);
        return xhr;
      },
      success: function (data, status, xhr) {
        loader_icon_off();
        temp.upload[this.id].progress = 100;

      }
    });
  }
}

function upload_status() {
  var stop = 1;

  conf.run.upload_status = 1;

  $.each(temp.upload, function (id, file) {
    if (!$('.autodoc_upload_status_' + id).length) {
      var obj = $(".autodoc_template_progress").children().clone();
      $(obj).addClass('autodoc_upload_status_' + id);
      $('.autodoc_upload_status').append(obj);
      stop = 0;
    }
    var obj = $('.autodoc_upload_status_' + id).children();
    if (file.error != null) {
      $(obj).html(file.obj.name + ': ' + file.error);
      $(obj).css('width', '100%');
      $(obj).addClass('bg-danger');
      $(obj).removeClass("progress-bar-animated");
      $(obj).removeClass("progress-bar-striped");

    }
    else if (file.progress == 100) {
      $(obj).html(file.obj.name);
      $(obj).css('width', '100%');
      $(obj).addClass('bg-success');
      $(obj).removeClass('progress-bar-animated');
      $(obj).removeClass("progress-bar-striped");
    }
    else if (file.progress < 50) {
      $(obj).html(file.obj.name + ': ' + 'reading file');
      $(obj).css('width', file.progress + '%');
      stop = 0;
    }
    else {
      $(obj).html(file.obj.name + ': ' + 'sending file');
      $(obj).css('width', file.progress + '%');
      stop = 0;
    }
  });

  if (stop) {
    conf.run.upload_status = 0;
  }
  else {
    setTimeout(function () { upload_status(); }, 100);
  }
}

function document_update_name(docid, name) {
  loader_icon_on();
  $.ajax({
    type: "PATCH",
    url: conf.openapi + "/documents/" + docid + "?name=" + encodeURIComponent(name),
    success: function (data) {
      loader_icon_off();
      $('span.autodoc_home_modal_name').removeClass('d-none');
      $('input.autodoc_home_modal_name').addClass('d-none');
      home_modal_open(docid);
    }
  });
}

// handle ajax busy icon
function loader_icon_on() {
  conf.busy++;
}

function loader_icon_off() {
  setTimeout(function() { conf.busy--; }, 1000);
}

function loader_icon_update() {
  if ( conf.busy > 0 ) {
    $('.autodoc_ajax_on').removeClass('d-none');
    $('.autodoc_ajax_off').addClass('d-none');
  }
  else {
    $('.autodoc_ajax_off').removeClass('d-none');
    $('.autodoc_ajax_on').addClass('d-none');
  }
  setTimeout(loader_icon_update, 100);
}